package syncloud.storage;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class StorageUtils {
    private IStorage storage;
    
    public StorageUtils(IStorage storage) {
        this.storage = storage;
    }

    private List<IFolder> getChilds(IFolder folder) {
        if (folder == null)
            return this.storage.getRoots();
        return folder.getFolders();
    }

    private IFolder createFolders(IFolder folder, List<String> pathComponents) {
        for (String missingComponent : pathComponents)
            folder = folder.createFolder(missingComponent);
        return folder;
    }

    public IFolder getOrCreateFolder(PathKey path) throws StorageException {
        IFolder found = null;

        List<String> pathComponents = new ArrayList<String>(path.getPathComponents());

        Iterator<String> iterator = pathComponents.iterator();
        while(iterator.hasNext()) {
            List<IFolder> folders = getChilds(found);
            IFolder newFound = FolderUtils.find(folders, iterator.next());
            if (newFound == null) break;
            iterator.remove();
            found = newFound;
        }

        return createFolders(found, pathComponents);
    }

    public IFolder getFolder(PathKey path) throws StorageException {
        IFolder found = null;

        for (String component : path.getPathComponents()) {
            List<IFolder> folders = getChilds(found);
            IFolder newFound = FolderUtils.find(folders, component);
            if (newFound == null) return null;
            found = newFound;
        }

        return found;
    }
    
    public List<IFolder> getFolders(PathKey path) throws StorageException {
        if (path == null) return null;
        IFolder folder = getFolder(path);
        return getChilds(folder);
    }
}
